package com.intershop.sellside.rest.mycoupon.v1.capi.request.mycoupon;

import java.util.function.Function;

import javax.inject.Inject;
import javax.inject.Provider;

import com.intershop.component.rest.capi.resource.RestResourceCacheHandler;
import com.intershop.sellside.rest.common.v1.capi.resourceobject.common.ContainerRO;
import com.intershop.sellside.rest.common.v1.capi.resourceobject.common.ContainerROBuilder;
import com.intershop.sellside.rest.mycoupon.v1.capi.handler.MyCouponFeedbackHandler;
import com.intershop.sellside.rest.mycoupon.v1.capi.handler.MyCouponHandler;
import com.intershop.sellside.rest.mycoupon.v1.capi.resourceobject.mycoupon.MyCouponRO;
import com.intershop.support.component.mycoupon.capi.MyCouponBO;

import jakarta.ws.rs.core.Response;
import jakarta.ws.rs.core.UriInfo;

/**
 * Request for "GET /mycoupons/&lt;coupon-id&gt;".
 */
public class MyCouponItemGetRequest
{
    private MyCouponHandler myCouponHandler;
    private RestResourceCacheHandler cacheHandler;
    private MyCouponFeedbackHandler myCouponFeedbackHandler;
    private Function<MyCouponBO, MyCouponRO> myCouponROMapper;
    private Provider<ContainerROBuilder<MyCouponRO>> containerROBuilderProvider;

    @Inject
    public MyCouponItemGetRequest(MyCouponHandler myCouponHandler, RestResourceCacheHandler cacheHandler,
                    MyCouponFeedbackHandler myCouponFeedbackHandler, Function<MyCouponBO, MyCouponRO> myCouponROMapper,
                    Provider<ContainerROBuilder<MyCouponRO>> containerROBuilderProvider)
    {
        this.myCouponHandler = myCouponHandler;
        this.cacheHandler = cacheHandler;
        this.myCouponFeedbackHandler = myCouponFeedbackHandler;
        this.myCouponROMapper = myCouponROMapper;
        this.containerROBuilderProvider = containerROBuilderProvider;
    }

    /**
     * Invokes the request with the given parameters.
     * 
     * @param uriInfo
     *            the URI information for the current request
     * @param couponCode
     *            the coupon code of to return details for
     * @return the response
     */
    public Response invoke(UriInfo uriInfo, String couponCode)
    {
        cacheHandler.setCacheExpires(0);

        MyCouponBO myCouponBO = myCouponHandler.getMyCouponByCode(couponCode);    
        
        if (myCouponBO == null)
        {
            return myCouponFeedbackHandler.getCouponNotFoundResponse();
        }

        MyCouponRO myCouponRO = myCouponROMapper.apply(myCouponBO);

        ContainerRO<MyCouponRO> containerRO = containerROBuilderProvider.get() //
                        .withData(myCouponRO) //
                        .withUriInfo(uriInfo) //
                        .build();

        return Response.ok(containerRO).build();
    }
}
