package com.intershop.sellside.rest.mycoupon.v1.capi.resource.mycoupon;

import static com.intershop.sellside.rest.mycoupon.v1.capi.APIConstants.API_NAME;
import static com.intershop.sellside.rest.mycoupon.v1.capi.APIConstants.API_VERSION;
import static com.intershop.sellside.rest.mycoupon.v1.capi.APIConstants.TAG_MYCOUPON;
import static com.intershop.sellside.rest.mycoupon.v1.capi.MyCouponConstantsREST.MEDIA_TYPE_MYCOUPON_V1_JSON;
import static com.intershop.sellside.rest.mycoupon.v1.capi.MyCouponConstantsREST.RESOURCE_PATH_MY_COUPON_V1;

import com.google.inject.Inject;
import com.google.inject.Injector;
import com.intershop.component.rest.capi.openapi.OpenAPIConstants;
import com.intershop.component.rest.capi.transaction.Transactional;
import com.intershop.sellside.rest.mycoupon.v1.capi.request.mycoupon.MyCouponListGetRequest;
import com.intershop.sellside.rest.mycoupon.v1.capi.request.mycoupon.MyCouponListPostRequest;
import com.intershop.sellside.rest.mycoupon.v1.capi.resourceobject.mycoupon.MyCouponRO;

import io.swagger.v3.oas.annotations.OpenAPIDefinition;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.extensions.Extension;
import io.swagger.v3.oas.annotations.extensions.ExtensionProperty;
import io.swagger.v3.oas.annotations.info.Info;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.QueryParam;
import jakarta.ws.rs.container.ResourceContext;
import jakarta.ws.rs.core.Context;
import jakarta.ws.rs.core.Response;
import jakarta.ws.rs.core.UriInfo;

/**
 * REST resource containing all operations for path "/mycoupons".
 */
@Tag(name = TAG_MYCOUPON)
@OpenAPIDefinition(info = @Info(version = API_VERSION), extensions = @Extension(
        properties = {@ExtensionProperty(name = OpenAPIConstants.API_ID, value = API_NAME)}))
@Path(RESOURCE_PATH_MY_COUPON_V1)
public class MyCouponListResource
{
    @Inject 
    private Injector injector;
    @Inject
    private MyCouponListGetRequest myCouponListGetRequest;
    @Inject
    private MyCouponListPostRequest myCouponListPostRequest;
    @Context
    private UriInfo uriInfo;
    @Context    
    private ResourceContext context;

    @Operation(summary = "Returns the available coupons.",
            description = "Returns the available coupons.")
    @ApiResponse(responseCode = "200", description = "The list of available coupons.",
            content = @Content(schema = @Schema(implementation = MyCouponRO.class)))
    @GET
    @Produces({MEDIA_TYPE_MYCOUPON_V1_JSON})
    public Response getAllMyCoupons_V1()
    {
        return myCouponListGetRequest.invoke(uriInfo);
    }
       
    /**
     * Returns for a given email a coupon.
     */
    @Operation(summary = "Creates a coupon.")
    @POST
    @Produces({MEDIA_TYPE_MYCOUPON_V1_JSON})
    @Transactional
    public Response generateMyCoupon_V1(@QueryParam("email") String email) 
    {
        return myCouponListPostRequest.invoke(uriInfo, email);
    }

    @Path("{couponId}")
    public MyCouponItemResource getMyCouponItemResource(@PathParam("couponId") String couponId)
    {
        MyCouponItemResource resource = new MyCouponItemResource(couponId);
        context.initResource(resource);
        injector.injectMembers(resource);
        return resource;
    }
}
