package com.intershop.sellside.rest.mycoupon.v1.internal.handler;

import java.util.Collection;
import java.util.Collections;

import com.google.inject.Inject;
import com.intershop.component.application.capi.ApplicationBO;
import com.intershop.component.application.capi.CurrentApplicationBOProvider;
import com.intershop.component.rest.capi.RestException;
import com.intershop.sellside.rest.mycoupon.v1.capi.handler.MyCouponHandler;
import com.intershop.support.component.mycoupon.capi.ApplicationBOMyCouponExtension;
import com.intershop.support.component.mycoupon.capi.MyCouponBO;
import com.intershop.support.component.mycoupon.capi.MyCouponBORepository;

public class MyCouponHandlerImpl implements MyCouponHandler
{
    @Inject
    private CurrentApplicationBOProvider currentApplicationBOProvider;

    @Override
    public Collection<MyCouponBO> getMyCouponBOs()
    {   
        MyCouponBORepository myCouponBORepository = getMyCouponBORepository();  
        if(myCouponBORepository==null)
        {
            return Collections.emptyList();
        }
        return myCouponBORepository.getAllCouponBOs();                        
    }

    @Override
    public MyCouponBO getMyCouponByCode(String couponId)
    {   
        MyCouponBORepository myCouponBORepository = getMyCouponBORepository();  
        if(myCouponBORepository==null)
        {
            return null;
        }   
        return myCouponBORepository.getMyCouponBOByCode(couponId);
    }
    
    @Override
    public MyCouponBO generateMyCoupon(String email)
    {
        // validate query parameter 'email'
        if (email == null || email.length() == 0)
        {
            throw new RestException().badRequest()
                            .message("Email is required to generate a coupon!")
                            .localizationKey("mycoupon.missing_email.error");
        }
        
        // get MyCouponBORepository to create a coupon
        MyCouponBORepository myCouponBORepository = getMyCouponBORepository();  
 
        // creates a coupon
        MyCouponBO myCouponBO = myCouponBORepository.createMyCouponBOByEmail(email);
        
        // validates the business object in case of error throw an exception
        if (myCouponBO == null || myCouponBO.getCode() == null)
        {
            throw new RestException().message("The Coupon code could not be generated!")
                            .localizationKey("mycoupon.generate_not_possible.error");
        }
        return myCouponBO;
    }
    
    /**
     * Returns for the given {@link ApplicationBO} the {@link MyCouponBORepository}
     * @return the {@link MyCouponBORepository}
     */
    private MyCouponBORepository getMyCouponBORepository()
    {
        ApplicationBOMyCouponExtension applicationBOMyCouponExtension = currentApplicationBOProvider.get()
                        .getExtension(ApplicationBOMyCouponExtension.EXTENSION_ID);
        return applicationBOMyCouponExtension.getMyCouponBORepository();
    }
}
