package com.intershop.sellside.rest.mycoupon.v1.capi.request.mycoupon;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.core.Is.is;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.util.function.Function;

import javax.inject.Inject;

import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnit;
import org.mockito.junit.MockitoRule;

import com.google.inject.AbstractModule;
import com.google.inject.Guice;
import com.google.inject.TypeLiteral;
import com.intershop.component.rest.capi.resource.RestResourceCacheHandler;
import com.intershop.sellside.rest.common.v1.capi.resourceobject.common.ContainerRO;
import com.intershop.sellside.rest.common.v1.capi.resourceobject.common.ContainerROBuilder;
import com.intershop.sellside.rest.common.v1.capi.resourceobject.feedback.FeedbackRO;
import com.intershop.sellside.rest.mycoupon.v1.capi.handler.MyCouponFeedbackHandler;
import com.intershop.sellside.rest.mycoupon.v1.capi.handler.MyCouponHandler;
import com.intershop.sellside.rest.mycoupon.v1.capi.resourceobject.mycoupon.MyCouponRO;
import com.intershop.sellside.rest.mycoupon.v1.internal.module.AppSfRestMyCouponRequestTestModule;
import com.intershop.support.component.mycoupon.capi.MyCouponBO;

import jakarta.ws.rs.core.Response;
import jakarta.ws.rs.core.UriInfo;

/**
 * Unit test for {@link MyCouponItemGetRequest}.
 */
public class MyCouponItemGetRequestTest
{
    private static final String MY_COUPON_ID = "testCouponId";

    @Rule public MockitoRule mockitoRule = MockitoJUnit.rule();

    @Mock private MyCouponHandler myCouponHandler;
    @Mock private RestResourceCacheHandler cacheHandler;
    @Mock private MyCouponFeedbackHandler myCouponFeedbackHandler;
    @Mock private Function<MyCouponBO, MyCouponRO> myCouponROMapper;
    @Mock private ContainerRO<MyCouponRO> containerRO;
    @Mock private UriInfo uriInfo;
    @Mock private MyCouponBO myCouponBO;
    @Mock private MyCouponRO myCouponRO;
    
    @Inject private ContainerROBuilder<MyCouponRO> containerROBuilder;

    @Inject private MyCouponItemGetRequest request;

    @Before
    public void init()
    {
        Guice.createInjector(new AppSfRestMyCouponRequestTestModule<>(new TypeLiteral<MyCouponRO>() {}), new AbstractModule()
        {
            @Override
            protected void configure()
            {
                bind(MyCouponHandler.class).toInstance(myCouponHandler);
                bind(RestResourceCacheHandler.class).toInstance(cacheHandler);
                bind(MyCouponFeedbackHandler.class).toInstance(myCouponFeedbackHandler);
                bind(new TypeLiteral<Function<MyCouponBO, MyCouponRO>>() {}).toInstance(myCouponROMapper);
            }
        }).injectMembers(this);
        when(containerROBuilder.build()).thenReturn(containerRO);

        when(myCouponHandler.getMyCouponByCode(MY_COUPON_ID)).thenReturn(myCouponBO);
        when(myCouponROMapper.apply(myCouponBO)).thenReturn(myCouponRO);
    }

    @Test
    public void testInvoke()
    {
        Response response = request.invoke(uriInfo, MY_COUPON_ID);

        assertThat(response.getStatus(), is(200));
        assertThat(response.getEntity(), is(containerRO));

        verify(containerROBuilder, never()).withError(any(FeedbackRO.class));
        verify(containerROBuilder, never()).withInfo(any(FeedbackRO.class));

        verify(cacheHandler).setCacheExpires(0);
    }

    @Test
    public void testInvoke_MyCouponNotFound()
    {
        Response expectedResponse = mock(Response.class);
        when(myCouponFeedbackHandler.getCouponNotFoundResponse()).thenReturn(expectedResponse);
        when(myCouponHandler.getMyCouponByCode(MY_COUPON_ID)).thenReturn(null);

        Response actualResponse = request.invoke(uriInfo, MY_COUPON_ID);

        assertThat(actualResponse, is(expectedResponse));
        verify(cacheHandler).setCacheExpires(0);
    }
}
