package com.intershop.sellside.rest.mycoupon.v1.internal.resource;

import static com.intershop.sellside.rest.mycoupon.v1.capi.MyCouponConstantsREST.RESOURCE_PATH_MY_COUPON_V1;
import static com.intershop.sellside.rest.mycoupon.v1.internal.resource.MyCouponListResourceProvider.MY_COUPON_RESOURCE_ACL_PATH;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.core.IsInstanceOf.instanceOf;
import static org.hamcrest.core.IsNull.notNullValue;
import static org.hamcrest.core.IsNull.nullValue;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnit;
import org.mockito.junit.MockitoRule;

import com.intershop.component.rest.capi.resource.RestResource;
import com.intershop.component.rest.capi.resource.RootResource;
import com.intershop.sellside.rest.mycoupon.v1.capi.resource.mycoupon.MyCouponListResource;

import jakarta.ws.rs.container.ResourceContext;

/**
 * Unit test for {@link MyCouponListResourceProvider}.
 */
public class MyCouponListResourceProviderTest
{
    @Rule
    public MockitoRule mockitoRule = MockitoJUnit.rule();

    @Mock
    private RootResource rootResource;
    @Mock
    private ResourceContext resourceContext;

    private TestMyCouponListResourceProvider provider;

    @Before
    public void init()
    {
        provider = new TestMyCouponListResourceProvider();
        when(rootResource.getRootResource()).thenReturn(rootResource);
    }

    @Test
    public void testGetSubResource()
    {
        Object resource = provider.getSubResource(rootResource, resourceContext, RESOURCE_PATH_MY_COUPON_V1);

        assertThat(resource, notNullValue());
        assertThat(resource, instanceOf(MyCouponListResource.class));
        verify(resourceContext).initResource(resource);
        verify(rootResource).addACLRuleIfSupported(MY_COUPON_RESOURCE_ACL_PATH);
    }

    @Test
    public void testGetSubResourceIfApplicable()
    {
        Object resource = provider.getSubResource(rootResource, resourceContext, RESOURCE_PATH_MY_COUPON_V1);

        assertThat(resource, notNullValue());
        assertThat(resource, instanceOf(MyCouponListResource.class));
        
        provider.isActive = false;
        
        resource = provider.getSubResource(rootResource, resourceContext, RESOURCE_PATH_MY_COUPON_V1);
        
        assertThat(resource, nullValue());
    }

    @Test
    public void testGetSubResource_WrongPath()
    {
        Object resource = provider.getSubResource(rootResource, resourceContext, "anotherPath");

        assertThat(resource, nullValue());
    }

    @Test
    public void testGetSubResource_WrongParent()
    {
        RestResource parentResource = mock(RestResource.class);
        when(parentResource.getRootResource()).thenReturn(rootResource);
        Object resource = provider.getSubResource(parentResource, resourceContext, MY_COUPON_RESOURCE_ACL_PATH);

        assertThat(resource, nullValue());
    }

    class TestMyCouponListResourceProvider extends MyCouponListResourceProvider
    {
        boolean isActive = true;

        @Override
        public boolean isApplicable(String cartridgeName)
        {
            return isActive;
        }
    }
}